import { RegexConstruct, RegexElement, RegexSequence } from '../types.mjs';
import '../encoder/types.mjs';

interface Capture extends RegexConstruct {
    type: 'capture';
    children: RegexElement[];
    options?: CaptureOptions;
}
type CaptureOptions = {
    /**
     * Name to be given to the capturing group.
     */
    name?: string;
};
interface Reference extends RegexConstruct {
    type: 'reference';
    name: string;
}
/**
 * Creates a capturing group which allows the matched pattern to be available:
 * - in the match results (`String.match`, `String.matchAll`, or `RegExp.exec`)
 * - in the regex itself, through {@link ref}
 */
declare function capture(sequence: RegexSequence, options?: CaptureOptions): Capture;
/**
 * Creates a reference, also known as backreference, which allows matching
 * again the exact text that a capturing group previously matched.
 *
 * In order to form a valid regex, the reference must use the same name as
 * a capturing group earlier in the expression.
 *
 * @param name - Name of the capturing group to reference.
 */
declare function ref(name: string): Reference;

export { type Capture, type CaptureOptions, type Reference, capture, ref };
