import { EncodeResult } from '../encoder/types.mjs';
import { RegexConstruct } from '../types.mjs';

/**
 * Character range from start to end (inclusive).
 */
interface CharacterRange {
    start: string;
    end: string;
}
interface CharacterClass extends RegexConstruct {
    type: 'characterClass';
    escape?: string;
    chars?: string[];
    ranges?: CharacterRange[];
    isNegated?: boolean;
}
/**
 * Matches any single character.
 * Cannot be used as a part of character class.
 */
declare const any: EncodeResult;
declare const digit: CharacterClass;
declare const nonDigit: CharacterClass;
declare const word: CharacterClass;
declare const nonWord: CharacterClass;
declare const whitespace: CharacterClass;
declare const nonWhitespace: CharacterClass;
/**
 * @deprecated Renamed to `nonDigit`.
 */
declare const notDigit: CharacterClass;
/**
 * @deprecated Renamed to `nonWord`.
 */
declare const notWord: CharacterClass;
/**
 * @deprecated Renamed to `nonWhitespace`.
 */
declare const notWhitespace: CharacterClass;
declare function charClass(...elements: CharacterClass[]): CharacterClass;
declare function charRange(start: string, end: string): CharacterClass;
declare function anyOf(characters: string): CharacterClass;
declare function negated(element: CharacterClass): CharacterClass;
/**
 * @deprecated Renamed to `negated`.
 */
declare const inverted: typeof negated;

export { type CharacterClass, type CharacterRange, any, anyOf, charClass, charRange, digit, inverted, negated, nonDigit, nonWhitespace, nonWord, notDigit, notWhitespace, notWord, whitespace, word };
