"use strict";
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);
var character_class_exports = {};
__export(character_class_exports, {
  any: () => any,
  anyOf: () => anyOf,
  charClass: () => charClass,
  charRange: () => charRange,
  digit: () => digit,
  inverted: () => inverted,
  negated: () => negated,
  nonDigit: () => nonDigit,
  nonWhitespace: () => nonWhitespace,
  nonWord: () => nonWord,
  notDigit: () => notDigit,
  notWhitespace: () => notWhitespace,
  notWord: () => notWord,
  whitespace: () => whitespace,
  word: () => word
});
module.exports = __toCommonJS(character_class_exports);
const any = {
  precedence: "atom",
  pattern: "."
};
const digit = {
  type: "characterClass",
  escape: "\\d",
  encode: encodeCharacterClass
};
const nonDigit = {
  type: "characterClass",
  escape: "\\D",
  encode: encodeCharacterClass
};
const word = {
  type: "characterClass",
  escape: "\\w",
  encode: encodeCharacterClass
};
const nonWord = {
  type: "characterClass",
  escape: "\\W",
  encode: encodeCharacterClass
};
const whitespace = {
  type: "characterClass",
  escape: "\\s",
  encode: encodeCharacterClass
};
const nonWhitespace = {
  type: "characterClass",
  escape: "\\S",
  encode: encodeCharacterClass
};
const notDigit = nonDigit;
const notWord = nonWord;
const notWhitespace = nonWhitespace;
function charClass(...elements) {
  if (elements.some((e) => e.isNegated)) {
    throw new Error("`charClass` should receive only non-negated character classes");
  }
  if (elements.length === 1) {
    return elements[0];
  }
  return {
    type: "characterClass",
    chars: elements.map((c) => getAllChars(c) ?? []).flat(),
    ranges: elements.map((c) => c.ranges ?? []).flat(),
    encode: encodeCharacterClass
  };
}
function charRange(start, end) {
  if (start.length !== 1) {
    throw new Error("`charRange` should receive only single character `start` string");
  }
  if (end.length !== 1) {
    throw new Error("`charRange` should receive only single character `end` string");
  }
  if (start > end) {
    throw new Error("`start` should be before or equal to `end`");
  }
  return {
    type: "characterClass",
    ranges: [{ start, end }],
    encode: encodeCharacterClass
  };
}
function anyOf(characters) {
  const chars = characters.split("").map((c) => escapeForCharacterClass(c));
  if (chars.length === 0) {
    throw new Error("`anyOf` should received at least one character");
  }
  return {
    type: "characterClass",
    chars,
    encode: encodeCharacterClass
  };
}
function negated(element) {
  return {
    type: "characterClass",
    escape: element.escape,
    chars: element.chars,
    ranges: element.ranges,
    isNegated: !element.isNegated,
    encode: encodeCharacterClass
  };
}
const inverted = negated;
function encodeCharacterClass() {
  if (this.escape === void 0 && !this.chars?.length && !this.ranges?.length) {
    throw new Error("Character class should contain at least one character or character range");
  }
  if (this.escape !== void 0 && !this.chars?.length && !this.ranges?.length && !this.isNegated) {
    return {
      precedence: "atom",
      pattern: this.escape
    };
  }
  const allChars = getAllChars(this) ?? [];
  const hyphen = allChars.includes("-") ? "-" : "";
  const caret = allChars.includes("^") ? "^" : "";
  const otherChars = allChars.filter((c) => c !== "-" && c !== "^").join("");
  const ranges = this.ranges?.map(({ start, end }) => `${start}-${end}`).join("") ?? "";
  const negation = this.isNegated ? "^" : "";
  let pattern = `[${negation}${ranges}${otherChars}${caret}${hyphen}]`;
  if (pattern === "[^-]")
    pattern = "[\\^-]";
  return {
    precedence: "atom",
    pattern
  };
}
function escapeForCharacterClass(text) {
  return text.replace(/[\]\\]/g, "\\$&");
}
function getAllChars(characterClass) {
  if (characterClass.escape === void 0) {
    return characterClass.chars;
  }
  return [characterClass.escape, ...characterClass.chars ?? []];
}
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  any,
  anyOf,
  charClass,
  charRange,
  digit,
  inverted,
  negated,
  nonDigit,
  nonWhitespace,
  nonWord,
  notDigit,
  notWhitespace,
  notWord,
  whitespace,
  word
});
//# sourceMappingURL=character-class.js.map