import { RegexConstruct, RegexElement, RegexSequence } from '../types.mjs';
import '../encoder/types.mjs';

/**
 * Positive lookahead assertion.
 *
 * A positive lookahead assertion is a zero-width assertion that matches a group of characters only if it is followed by a specific group of characters.
 *
 * @example
 * ```ts
 * lookahead("a");
 * // /(?=a)/
 *
 * lookahead(["a", "b", "c"]);
 * // /(?=abc)/
 * ```
 */
interface Lookahead extends RegexConstruct {
    type: 'lookahead';
    children: RegexElement[];
}
declare function lookahead(sequence: RegexSequence): Lookahead;

export { type Lookahead, lookahead };
