import { RegexConstruct, RegexElement, RegexSequence } from '../types.js';
import '../encoder/types.js';

/**
 * Positive lookbehind assertion.
 *
 * A positive lookbehind assertion is a zero-width assertion that matches a group of characters only if it is preceded by a specific group of characters.
 *
 * @example
 * ```ts
 * lookbehind("a");
 * // /(?<=a)/
 *
 * lookbehind(["a", "b", "c"]);
 * // /(?<=abc)/
 * ```
 */
interface Lookbehind extends RegexConstruct {
    type: 'lookbehind';
    children: RegexElement[];
}
declare function lookbehind(sequence: RegexSequence): Lookbehind;

export { type Lookbehind, lookbehind };
