import { RegexConstruct, RegexElement, RegexSequence } from '../types.js';
import '../encoder/types.js';

/**
 * Negative lookahead assertion.
 *
 * A negative lookahead assertion is a zero-width assertion that matches a group of characters only if it is not followed by a specific group of characters.
 *
 * @example
 * ```ts
 * negativeLookahead("a");
 * // /(?=a)/
 *
 * negativeLookahead(["a", "b", "c"]);
 * // /(?=abc)/
 * ```
 */
interface NegativeLookahead extends RegexConstruct {
    type: 'negativeLookahead';
    children: RegexElement[];
}
declare function negativeLookahead(sequence: RegexSequence): NegativeLookahead;

export { type NegativeLookahead, negativeLookahead };
