import { RegexConstruct, RegexElement, RegexSequence } from '../types.mjs';
import '../encoder/types.mjs';

/**
 * Negative lookbehind assertion.
 *
 * A negative lookbehind assertion is a zero-width assertion that matches a group of characters only if it is not preceded by a specific group of characters.
 *
 * @example
 * ```ts
 * negativeLookbehind("a");
 * // /(?<!a)/
 *
 * negativeLookbehind(["a", "b", "c"]);
 * // /(?<!abc)/
 * ```
 */
interface NegativeLookbehind extends RegexConstruct {
    type: 'negativeLookbehind';
    children: RegexElement[];
}
declare function negativeLookbehind(sequence: RegexSequence): NegativeLookbehind;

export { type NegativeLookbehind, negativeLookbehind };
