"use strict";
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);
var encoder_exports = {};
__export(encoder_exports, {
  encodeAtom: () => encodeAtom,
  encodeSequence: () => encodeSequence
});
module.exports = __toCommonJS(encoder_exports);
var import_text = require("../utils/text");
function encodeSequence(elements) {
  const encodedNodes = elements.map((n) => encodeNode(n));
  return concatSequence(encodedNodes);
}
function encodeAtom(elements) {
  return wrapAtom(encodeSequence(elements));
}
function encodeNode(element) {
  if (typeof element === "string") {
    return encodeText(element);
  }
  if (typeof element === "object" && element instanceof RegExp) {
    return encodeRegExp(element);
  }
  if (typeof element === "object" && "pattern" in element) {
    return element;
  }
  if (typeof element === "object" && typeof element.encode !== "function") {
    throw new Error(`\`encodeNode\`: unknown element type ${element.type}`);
  }
  return element.encode();
}
function encodeText(text) {
  if (text.length === 0) {
    throw new Error("`encodeText`: received text should not be empty");
  }
  if (text.length === 1) {
    return {
      precedence: "atom",
      pattern: (0, import_text.escapeText)(text)
    };
  }
  return {
    precedence: "sequence",
    pattern: (0, import_text.escapeText)(text)
  };
}
function encodeRegExp(regexp) {
  const pattern = regexp.source;
  return {
    precedence: isAtomicPattern(pattern) ? "atom" : "disjunction",
    pattern
  };
}
function isAtomicPattern(pattern) {
  if (pattern.length === 1) {
    return true;
  }
  if (pattern.startsWith("[") && pattern.endsWith("]") && pattern.match(/[[\]]/g)?.length === 2) {
    return true;
  }
  if (pattern.startsWith("(") && pattern.endsWith(")") && pattern.match(/[()]/g)?.length === 2) {
    return true;
  }
  return false;
}
function concatSequence(encoded) {
  if (encoded.length === 1) {
    return encoded[0];
  }
  return {
    precedence: "sequence",
    pattern: encoded.map((n) => (n.precedence === "disjunction" ? wrapAtom(n) : n).pattern).join("")
  };
}
function wrapAtom(encoded) {
  if (encoded.precedence === "atom") {
    return encoded;
  }
  return {
    precedence: "atom",
    pattern: `(?:${encoded.pattern})`
  };
}
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  encodeAtom,
  encodeSequence
});
//# sourceMappingURL=encoder.js.map