import { escapeText } from "../utils/text";
function encodeSequence(elements) {
  const encodedNodes = elements.map((n) => encodeNode(n));
  return concatSequence(encodedNodes);
}
function encodeAtom(elements) {
  return wrapAtom(encodeSequence(elements));
}
function encodeNode(element) {
  if (typeof element === "string") {
    return encodeText(element);
  }
  if (typeof element === "object" && element instanceof RegExp) {
    return encodeRegExp(element);
  }
  if (typeof element === "object" && "pattern" in element) {
    return element;
  }
  if (typeof element === "object" && typeof element.encode !== "function") {
    throw new Error(`\`encodeNode\`: unknown element type ${element.type}`);
  }
  return element.encode();
}
function encodeText(text) {
  if (text.length === 0) {
    throw new Error("`encodeText`: received text should not be empty");
  }
  if (text.length === 1) {
    return {
      precedence: "atom",
      pattern: escapeText(text)
    };
  }
  return {
    precedence: "sequence",
    pattern: escapeText(text)
  };
}
function encodeRegExp(regexp) {
  const pattern = regexp.source;
  return {
    precedence: isAtomicPattern(pattern) ? "atom" : "disjunction",
    pattern
  };
}
function isAtomicPattern(pattern) {
  if (pattern.length === 1) {
    return true;
  }
  if (pattern.startsWith("[") && pattern.endsWith("]") && pattern.match(/[[\]]/g)?.length === 2) {
    return true;
  }
  if (pattern.startsWith("(") && pattern.endsWith(")") && pattern.match(/[()]/g)?.length === 2) {
    return true;
  }
  return false;
}
function concatSequence(encoded) {
  if (encoded.length === 1) {
    return encoded[0];
  }
  return {
    precedence: "sequence",
    pattern: encoded.map((n) => (n.precedence === "disjunction" ? wrapAtom(n) : n).pattern).join("")
  };
}
function wrapAtom(encoded) {
  if (encoded.precedence === "atom") {
    return encoded;
  }
  return {
    precedence: "atom",
    pattern: `(?:${encoded.pattern})`
  };
}
export {
  encodeAtom,
  encodeSequence
};
//# sourceMappingURL=encoder.mjs.map